
import logging
import sys
import time

from publicaddr import google as PROVIDER_GOOGLE
from publicaddr import opendns as PROVIDER_OPENDNS
from publicaddr import cloudflare as PROVIDER_CLOUDFLARE
from publicaddr import akamai as PROVIDER_AKAMAI
from publicaddr import ipify as PROVIDER_IPIFY
from publicaddr import icanhazip as PROVIDER_ICANHAZIP
from publicaddr import matrix as PROVIDER_MATRIX

from publicaddr import randprov
from publicaddr import constants

from publicaddr.constants import *

loglevel = logging.CRITICAL
logging.basicConfig(format='%(levelname)s %(message)s', stream=sys.stdout, level=loglevel)
logging.getLogger("requests").setLevel(loglevel)
logging.getLogger("urllib3").setLevel(loglevel)

# register providers
randprov.set_providers([PROVIDER_GOOGLE, PROVIDER_OPENDNS, PROVIDER_IPIFY,
                        PROVIDER_CLOUDFLARE, PROVIDER_AKAMAI, PROVIDER_ICANHAZIP,
                        PROVIDER_MATRIX])

# get all public IP if exists
def lookup(providers=constants.ALL_PROVIDERS, retries=3):
    """lookup your public ipv4 and ipv6 from random providers"""
    addrs = {}

    for _ in range(retries):
        start = time.perf_counter()

        # select provider in random mode
        if providers == constants.ALL_PROVIDERS: 
            _provider = randprov.pick_all()
            if len(_provider.dns_servers) and not len(_provider.http_servers) and not len(_provider.stuns_servers): 
                _ipproto = constants.PROTO_DNS
            elif not len(_provider.dns_servers) and len(_provider.http_servers) and not len(_provider.stuns_servers): 
                _ipproto = constants.PROTO_HTTP
            elif not len(_provider.dns_servers) and not len(_provider.http_servers) and len(_provider.stuns_servers): 
                _ipproto = constants.PROTO_STUNS
            else:
                _ipproto = randprov.pick_proto()

        elif providers == constants.DNS_PROVIDERS: 
            _provider = randprov.pick_dns()
            _ipproto = constants.PROTO_DNS

        elif providers == constants.HTTP_PROVIDERS: 
            _provider = randprov.pick_http()
            _ipproto = constants.PROTO_HTTP

        elif providers == constants.STUNS_PROVIDERS: 
            _provider = randprov.pick_stuns()
            _ipproto = constants.PROTO_STUNS

        else:
            logging.error("fetch invalid providers mode - %s" % providers)
            return addrs

        # lookup for public all ip
        addrs["ip4"] = _provider.lookup(ipversion=constants.IP_V4, ipproto=_ipproto)
        addrs["ip6"] = _provider.lookup(ipversion=constants.IP_V6, ipproto=_ipproto)
        addrs["provider"] = _provider.NAME
        addrs["proto"] = constants.PROTO_STR[_ipproto]

        request_time = time.perf_counter() - start
        addrs["duration"] = "{0:.3f}".format(request_time)

        if not(addrs["ip4"] is None and addrs["ip6"] is None):
            break

    return addrs

# return from a specific provider, the IPv4 or IPv6
def get(provider=PROVIDER_GOOGLE, ipversion=constants.IP_V4, ipproto=constants.PROTO_DNS):
    """return your public ipv4 or ipv6"""
    start = time.perf_counter()
    addr = {}
    addr["ip"] = provider.lookup(ipversion=ipversion, ipproto=ipproto)
    request_time = time.perf_counter() - start
    addr["duration"] = "{0:.3f}".format(request_time)
    return addr




import requests
import logging

from publicaddr import constants

NAME = "Icanhazip"

timeout = 1

dns_servers = {}

http_servers = {
    "ip4": "https://ipv4.icanhazip.com/",
    "ip6": "https://ipv6.icanhazip.com/",
}

stuns_servers = {}

def lookup_http(ipversion):
    ip = None
    try:
        if ipversion == 4:
            requests.packages.urllib3.util.connection.HAS_IPV6 = False
            ip = requests.get(http_servers["ip4"], timeout=timeout).text.rstrip()
        if ipversion == 6:
            requests.packages.urllib3.util.connection.HAS_IPV6 = True
            ip = requests.get(http_servers["ip6"], timeout=timeout).text.rstrip()
    except requests.exceptions.RequestException as e:
        logging.error("icanhazip unable to get ip info - %s" % e)
    return ip

def lookup(ipversion, ipproto):
    ret = None

    if ipproto == constants.PROTO_HTTP:
        ret = lookup_http(ipversion)
        
    else:
        logging.error("icanhazip lookup invalid ipproto - %s" % ipproto)
           
    return ret



import logging
import requests
import aiostun
import dns.resolver
import dns.exception
import asyncio

from publicaddr import constants

NAME = "Google"

timeout = 5

 # ns1.google.com, ns2.google.com, ns3.google.com
dns_servers = {
    "ip4": ["216.239.32.10", "216.239.34.10", "216.239.36.10"],
    "ip6": ["2001:4860:4802:32::a", "2001:4860:4802:34::a", "2001:4860:4802:36::a"],
}

http_servers = {
    "ip4": "https://domains.google.com/checkip",
    "ip6": "https://domains.google.com/checkip",
}

stuns_servers = {
    "ip4": "turns.goog",
    "ip6": "turns.goog",
}

def _resolv_addr(nameservers=[], qname="o-o.myaddr.google.com", rdtype="TXT" ):
    dnsresolv = dns.resolver.Resolver(configure=False)
    dnsresolv.nameservers = nameservers
    dnsresolv.timeout = timeout
    dnsresolv.lifetime = timeout

    # make dns resolution
    answers = dnsresolv.resolve(qname, rdtype)
    return answers[0].strings[0].decode()

def lookup_dns(ipversion):
    ip = None
    try:
        if ipversion == 4:
            ip = _resolv_addr(nameservers=dns_servers["ip4"])
        if ipversion == 6:
            ip = _resolv_addr(nameservers=dns_servers["ip6"])
    except dns.exception.DNSException as e:
        logging.error("google unable to get dns ip info - %s" % e)
    return ip

def lookup_http(ipversion):
    ip = None
    try:
        if ipversion == 4:
            requests.packages.urllib3.util.connection.HAS_IPV6 = False
            ip = requests.get(http_servers["ip4"], timeout=timeout).text.rstrip()
        if ipversion == 6:
            requests.packages.urllib3.util.connection.HAS_IPV6 = True
            ip = requests.get(http_servers["ip6"], timeout=timeout).text.rstrip()
    except requests.exceptions.RequestException as e:
        logging.error("google unable to get http ip info - %s" % e)
    return ip

async def _lookup_stun(ipversion):
    if ipversion == 4:
        async with aiostun.Client(host=stuns_servers["ip4"], port=443, family=aiostun.IP4, proto=aiostun.TLS) as stunc:
            mapped_addr = await stunc.get_mapped_address()
    if ipversion == 6:
        async with aiostun.Client(host=stuns_servers["ip6"], port=443, family=aiostun.IP6, proto=aiostun.TLS) as stunc:
            mapped_addr = await stunc.get_mapped_address()
    return mapped_addr

def lookup_stun(ipversion):
    ip = None
    try:
        mapped_addr = asyncio.run(_lookup_stun(ipversion))
        ip = mapped_addr["ip"]
    except Exception as e:
        logging.error("google unable to get stun ip info - %s" % e)
    return ip

def lookup(ipversion, ipproto):
    ret = None

    if ipproto == constants.PROTO_DNS:
        ret = lookup_dns(ipversion)

    elif ipproto == constants.PROTO_HTTP:
        ret = lookup_http(ipversion)
        
    elif ipproto == constants.PROTO_STUNS:
        ret = lookup_stun(ipversion)

    else:
        logging.error("google lookup invalid ipproto - %s" % ipproto)

    return ret



import requests
import dns.resolver
import dns.exception
import logging

from publicaddr import constants

NAME = "Akamai"

timeout = 1

# a1-67.akam.net, a3-67.akam.net
dns_servers = {
    "ip4": ["193.108.91.67", "96.7.49.67"],
    "ip6": ["2600:1401:2::43", "2600:1408:1c::43"]
}

http_servers = {
    "ip4": "http://ipv4.whatismyip.akamai.com/",
    "ip6": "http://ipv6.whatismyip.akamai.com/",
}

stuns_servers = {}

# dig @193.108.91.67 whoami.ds.akahelp.net TXT +short
# "ns" "x.x.x.x"
def _resolv_addr(nameservers=[], qname="whoami.ds.akahelp.net", rdtype="TXT"):
    dnsresolv = dns.resolver.Resolver(configure=False)
    dnsresolv.nameservers = nameservers
    dnsresolv.timeout = timeout
    dnsresolv.lifetime = timeout

    answers = dnsresolv.resolve(qname, rdtype)
    return answers[0].strings[1].decode()

def lookup_dns(ipversion):
    ip = None
    try:
        if ipversion == 4:
            ip = _resolv_addr(nameservers=dns_servers["ip4"])
        if ipversion == 6:
            ip = _resolv_addr(nameservers=dns_servers["ip6"])
    except dns.exception.DNSException as e:
        logging.error("akamai unable to get ip info - %s" % e)
    return ip

def lookup_http(ipversion):
    ip = None
    try:
        if ipversion == 4:
            requests.packages.urllib3.util.connection.HAS_IPV6 = False
            ip = requests.get(http_servers["ip4"], timeout=timeout).text.rstrip()
        if ipversion == 6:
            requests.packages.urllib3.util.connection.HAS_IPV6 = True
            ip = requests.get(http_servers["ip6"], timeout=timeout).text.rstrip()
    except requests.exceptions.RequestException as e:
        logging.error("akamai unable to get ip info - %s" % e)
    return ip

def lookup(ipversion, ipproto):
    ret = None

    if ipproto == constants.PROTO_DNS:
        ret = lookup_dns(ipversion)

    elif ipproto == constants.PROTO_HTTP:
        ret = lookup_http(ipversion)
        
    else:
        logging.error("akamai lookup invalid ipproto - %s" % ipproto)
           
    return ret



import logging

import dns.resolver
import dns.exception

from publicaddr import constants

NAME = "OpenDNS"

timeout = 1

# resolver1.opendns.com, resolver2.opendns.com
dns_servers = {
    "ip4": ["208.67.222.222", "208.67.220.220"],
    "ip6": ["2620:119:35::35", "2620:119:53::53"],
}

http_servers = {}

stuns_servers = {}

# dig myip.opendns.com @resolver1.opendns.com -6 AAAA +short
# dig myip.opendns.com @resolver1.opendns.com -4 A +short

def _resolv_addr(nameservers=[], qname="myip.opendns.com", rdtype="A"):
    dnsresolv = dns.resolver.Resolver(configure=False)
    dnsresolv.nameservers = nameservers
    dnsresolv.timeout = timeout
    dnsresolv.lifetime = timeout
    
    # make dns resolution
    answers = dnsresolv.resolve(qname, rdtype)
    return answers[0].to_text()

def lookup_dns(ipversion):
    ip = None
    try:
        if ipversion == 4:
            ip  = _resolv_addr(nameservers=dns_servers["ip4"], rdtype="A")
        if ipversion == 6:
            ip = _resolv_addr(nameservers=dns_servers["ip6"], rdtype="AAAA")
    except dns.exception.DNSException as e:
        logging.error("opendns unable to get ip info - %s" % e)
    return ip

def lookup(ipversion, ipproto):
    ret = None

    if ipproto == constants.PROTO_DNS:
        ret = lookup_dns(ipversion)
        
    else:
        logging.error("opendns lookup invalid ipproto - %s" % ipproto)
           
    return ret



import logging
import requests
import aiostun
import dns.resolver
import dns.exception
import asyncio

from publicaddr import constants

NAME = "Matrix"

timeout = 5

dns_servers = {}

http_servers = {}

stuns_servers = {
    "ip4": "turn.matrix.org",
    "ip6": "turn.matrix.org",
}

async def _lookup_stun(ipversion):
    if ipversion == 4:
        async with aiostun.Client(host=stuns_servers["ip4"], port=443, family=aiostun.IP4, proto=aiostun.TLS) as stunc:
            mapped_addr = await stunc.get_mapped_address()
    if ipversion == 6:
        async with aiostun.Client(host=stuns_servers["ip6"], port=443, family=aiostun.IP6, proto=aiostun.TLS) as stunc:
            mapped_addr = await stunc.get_mapped_address()
    return mapped_addr

def lookup_stun(ipversion):
    ip = None
    try:
        mapped_addr = asyncio.run(_lookup_stun(ipversion))
        ip = mapped_addr["ip"]
    except Exception as e:
        logging.error("matrix unable to get stun ip info - %s" % e)
    return ip

def lookup(ipversion, ipproto):
    ret = None

    if ipproto == constants.PROTO_STUNS:
        ret = lookup_stun(ipversion)

    else:
        logging.error("matrix lookup invalid ipproto - %s" % ipproto)

    return ret


# Use IPv4 or IPv6 protocols to reach the provider
IP_V4=4
IP_V6=6

# Use DNS or HTTP protocols to reach the provider
PROTO_DNS  = 1
PROTO_HTTP = 2
PROTO_STUNS = 3
PROTO_STR = {1: "DNS", 2: "HTTP", 3: "STUNS"}

# Providers by type
ALL_PROVIDERS = 0
DNS_PROVIDERS = 1
HTTP_PROVIDERS = 2
STUNS_PROVIDERS = 3



from random import randrange

_providers = []
_providers_dns = []
_providers_http = []
_providers_stuns = []

def set_providers(providers):
    """set all providers"""
    global _providers
    global _providers_dns
    global _providers_http
    global _providers_stuns
    _providers = providers

    for prov in providers:
        if len(prov.dns_servers): _providers_dns.append(prov)
        if len(prov.http_servers): _providers_http.append(prov)
        if len(prov.stuns_servers): _providers_stuns.append(prov)

def pick_proto():
    return randrange(3)+1

def pick_all():
    global _providers 
    id = randrange(len(_providers))
    return _providers[id]

def pick_dns():
    global _providers_dns
    id = randrange(len(_providers_dns))
    return _providers_dns[id]

def pick_http():
    global _providers_http
    id = randrange(len(_providers_http))
    return _providers_http[id]

def pick_stuns():
    global _providers_stuns
    id = randrange(len(_providers_stuns))
    return _providers_stuns[id]



import unittest
import publicaddr

class TestAkamai(unittest.TestCase):
    def test_get_dns_ip4(self):
        """get dns ip4"""
        ip = publicaddr.get(provider=publicaddr.PROVIDER_AKAMAI, ipversion=publicaddr.IP_V4)
        self.assertNotEqual(ip, "")
    def test_get_dns_ip6(self):
        """get dns ip6"""
        ip = publicaddr.get(provider=publicaddr.PROVIDER_AKAMAI, ipversion=publicaddr.IP_V6)
        self.assertNotEqual(ip, "")
    def test_get_ip4_http(self):
        """get dns ip4"""
        ip = publicaddr.get(provider=publicaddr.PROVIDER_AKAMAI, ipversion=publicaddr.IP_V4, ipproto=publicaddr.PROTO_HTTP)
        self.assertNotEqual(ip, "")
    def test_get_ip6_http(self):
        """get dns ip6"""
        ip = publicaddr.get(provider=publicaddr.PROVIDER_AKAMAI, ipversion=publicaddr.IP_V6, ipproto=publicaddr.PROTO_HTTP)
        self.assertNotEqual(ip, "")
    def test_lookup(self):
        """get dns ip6 and ip4"""
        ips = publicaddr.lookup()
        self.assertIn("ip4", ips)
        self.assertIn("ip6", ips)




import dns.resolver
import dns.exception
import logging

from publicaddr import constants

NAME = "Cloudflare"

timeout = 1

# resolver1.opendns.com, resolver2.opendns.com
dns_servers = {
    "ip4": ["1.1.1.1", "1.0.0.1"],
    "ip6": ["2606:4700:4700::1111", "2606:4700:4700::1001"],
}

http_servers = {}

stuns_servers = {}

# dig @2606:4700:4700::1111 whoami.cloudflare TXT CH +short
# dig @2606:4700:4700::1111 whoami.cloudflare TXT CH +short
def _resolv_addr(nameservers=[], qname="whoami.cloudflare", rdtype="TXT", rdclass="CH"  ):
    dnsresolv = dns.resolver.Resolver(configure=False)
    dnsresolv.nameservers = nameservers
    dnsresolv.timeout = timeout
    dnsresolv.lifetime = timeout
    
    # make dns resolution
    answers = dnsresolv.resolve(qname, rdtype, rdclass)
    return answers[0].strings[0].decode()

def lookup_dns(ipversion):
    ip = None
    try:
        if ipversion == 4:
            ip = _resolv_addr(nameservers=dns_servers["ip4"])
        if ipversion == 6:
            ip = _resolv_addr(nameservers=dns_servers["ip6"])
    except dns.exception.DNSException as e:
        logging.error("cloudflare unable to get ip info - %s" % e)
    return ip

def lookup(ipversion, ipproto):
    ret = None

    if ipproto == constants.PROTO_DNS:
        ret = lookup_dns(ipversion)
        
    else:
        logging.error("cloudflare lookup invalid ipproto - %s" % ipproto)
           
    return ret